package geniusweb.events;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.junit.Test;
import org.junit.experimental.theories.DataPoint;
import org.junit.experimental.theories.DataPoints;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.actions.PartyId;
import tudelft.utilities.junit.GeneralTests;

public class SessionStartedTest extends GeneralTests<SessionStarted> {

	private final ObjectMapper jackson = new ObjectMapper();

	private static final Long SESSIONNR = 1l;
	protected static final Long NOW = (long) 101;
	protected static final Long NOW1 = NOW + 1;

	private static final PartyId PARTY1 = new PartyId("party1");
	private static final PartyId PARTY2 = new PartyId("party2");

	private static List<PartyId> parties = Arrays.asList(PARTY1, PARTY2);

	private static final SessionStarted sessionStartedLater = new SessionStarted(
			SESSIONNR, parties, NOW1);
	private static final SessionStarted othersessionStarted = new SessionStarted(
			SESSIONNR + 1, parties, NOW);

	public static SessionStarted sessionStarted = new SessionStarted(SESSIONNR,
			parties, NOW);
	private static final SessionStarted sessionStarted1 = new SessionStarted(
			SESSIONNR, parties, NOW);
	// jackson expected format.
	private static String sessionstartedstring = "{\"SessionStarted\":{\"sessionNumber\":1,\"parties\":[\"party1\",\"party2\"],\"time\":101}}";
	// "{\"sessionstarted\":{\"time\":101,\"sessionNumber\":1,\"parties\":[\"party1\",\"party2\"]}}";
	public static String expectedstring = "SessionStarted.*101.*";

	@Override
	public List<List<SessionStarted>> getGeneralTestData() {
		List<List<SessionStarted>> list = new LinkedList<>();
		list.add(Arrays.asList(sessionStarted, sessionStarted1));
		list.add(Arrays.asList(sessionStartedLater));
		list.add(Arrays.asList(othersessionStarted));
		return list;
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList("SessionStarted.*" + SESSIONNR + ".*" + NOW + ".*",
				"SessionStarted.*" + SESSIONNR + ".*" + NOW1 + ".*",
				"SessionStarted.*" + (SESSIONNR + 1) + ".*" + NOW + ".*");
	}

	@DataPoints("equal")
	public static SessionStarted[] equal() {
		return new SessionStarted[] { sessionStarted, sessionStarted1 };
	}

	@DataPoints("notequal")
	public static SessionStarted[] notequal() {
		return new SessionStarted[] { sessionStartedLater,
				sessionStartedLater };
	}

	@DataPoint("equalstring")

	@Test
	public void serializeStartedTest() throws JsonProcessingException {
		System.out.println(jackson.writeValueAsString(sessionStarted));
		assertEquals(sessionstartedstring,
				jackson.writeValueAsString(sessionStarted));
	}

	@Test
	public void deserializeStartedTest() throws IOException {
		NegotiationEvent evt = jackson.readValue(sessionstartedstring,
				NegotiationEvent.class);
		assertEquals(SessionStarted.class, evt.getClass());
		SessionStarted event = (SessionStarted) evt;
		// compare fields, as sessionended is a derived/new inner class
		assertEquals(NOW, event.getTime());
		assertEquals(SESSIONNR, event.getSessionNumber());
		assertEquals(2, event.getParties().size());
	}

}
